

extends KinematicBody2D

# member variables here, example:
# var a=2
# var b="textvar"


var Config = preload("res://global/config.gd")

const FLOOR_ANGLE_TOLERANCE = 30
var JUMP_VELOCITY = Config.BEAN_JUMP_VELOCITY
var POGO_JUMP_VELOCITY = Config.BEAN_POGO_JUMP_VELOCITY
var GRAVITY = Config.BEAN_GRAVITY
var JUMP_UMBRELLA_VELOCITY = Config.BEAN_JUMP_UMBRELLA_VELOCITY
var FALL_UMBRELLA_VELOCITY = Config.BEAN_FALL_UMBRELLA_VELOCITY
var BALLOON_LIFT=Config.BEAN_BALLOON_LIFT
var BALLOON_FALL=Config.BEAN_BALLOON_FALL
const MAX_ONFLOOR_TIME=0.1
var TARGET_SPEED = Config.BEAN_TARGET_SPEED
var CAR_TARGET_SPEED = Config.BEAN_CAR_TARGET_SPEED
var SLOWDOWN_SPEED = Config.BEAN_SLOWDOWN_SPEED
var JUMP_STOP_FORCE = Config.BEAN_JUMP_STOP_FORCE
var WIND_UP_SPEED = Config.BEAN_WIND_UP_SPEED
var COINS_PER_BEAN_RUSH = Config.COIN_COUNT_BEAN_RUSH
var first_hit = false
var disable_jump=false
var stopped=false
var ACCEL= Config.BEAN_ACCEL
const MAX_CAR_CRASHES=2

var level_ended=false
var bean_count=0
var bean_count_br=0
var bean_rush_count=0
var speed_mult=1.0

var invincible_time=0
var untrip_time=0
var blocked=false

const INVINCIBLE_MAX=3.0
const UNTRIP_MAX=0.6

const POWERUP_NONE = 0
const POWERUP_POGO = 1
const POWERUP_BALLOON = 2
const POWERUP_CAR = 3
const POWERUP_MAX = 4

var distance_run=0

var powerup_times=[0,
	Config.POWERUP_TIME_POGO,
	Config.POWERUP_TIME_BALLOON,
	Config.POWERUP_TIME_CAR
]

var powerup_chances=[0,
	Config.POWERUP_CHANCES_POGO,
	Config.POWERUP_CHANCES_BALLOON,
	Config.POWERUP_CHANCES_CAR
]
	
var powerup_time=0
var powerup_chance=0

var car_crashes=MAX_CAR_CRASHES
var current_powerup = POWERUP_NONE
var globals=null

var jumping=false

var vvelocity = 0.0

var target_speed=TARGET_SPEED
var speed = 300

var MOUSE_MIN_AIR_TIME=0.2

var base_anim=""
var umbrella_fall=false
var using_umbrella=false
var air_time=0
var mouse_pressed=false
var mouse_pressing=false

var jump_air_time=0
var jump_press=false

var press_pos=Vector2()

func _input(ev):
	if (level_ended):
		return
		
	if (ev.type==InputEvent.MOUSE_BUTTON and ev.pressed and ev.button_index==BUTTON_LEFT):
		mouse_pressed=true
		jump_press=false
		mouse_pressing=true
		press_pos=ev.pos/get_viewport_rect().size
	if (ev.type==InputEvent.MOUSE_BUTTON and not ev.pressed and ev.button_index==BUTTON_LEFT):
		mouse_pressing=false
		jump_press=false
		var rel_pos = ev.pos/get_viewport_rect().size
		if (rel_pos.x-press_pos.x > 0.2):
			_bean_rush()
	if (ev.type==InputEvent.KEY and ev.scancode==KEY_F1 and ev.pressed):
		fill_bean_rush()
		
			
		

var hat=null
var hat_name=""

func _update_hat():
	
	if (not hat || hat.get_sprite_frames()==null):
		return
		
	hat.set_frame( get_node("frames").get_frame() )
	hat.set_offset( get_node("frames").get_offset() )

func _process(delta):
	#_update_hat()
	pass

func _fixed_process(delta):

	if (delta>0.5):
		delta=0.5 # fix missing floor bugs?

	#_update_hat()
	
	if (stopped):
		
		vvelocity+=delta*GRAVITY
		var velocity = Vector2(0,vvelocity)		
		var motion = velocity * delta
		move(motion)
		return
		
	
	if (powerup_time>0):
		powerup_time-=delta		
		if (powerup_time<=0):
			### RESET LONGER CAR BOOST
			if(get_node("/root/globals").longer_car_boost):
				get_node("/root/globals").longer_car_boost = false
			
			current_powerup=POWERUP_NONE
			get_node("car_fx").stop_all()
			base_anim=""
			get_node("anim").play(base_anim+"run")
			jumping=false
			invincible_time=INVINCIBLE_MAX
			get_node("frames/hat").show()
			get_node("smoke").set_emitting(false)
	
	if (invincible_time>0):
		if (fmod(invincible_time,0.2)>0.1):
			get_node("frames").show()
		else:
			get_node("frames").hide()
			
		invincible_time-=delta
		
		if (invincible_time<=0):
			get_node("frames").show()
	
	if (untrip_time>0):		
		untrip_time-=delta

	if (current_powerup==POWERUP_CAR):
	
		if (speed<CAR_TARGET_SPEED*speed_mult):
			speed+=delta*ACCEL
		if (speed>CAR_TARGET_SPEED*speed_mult):
			speed=CAR_TARGET_SPEED
	else:

		if (speed<SLOWDOWN_SPEED*speed_mult):
			speed=SLOWDOWN_SPEED*speed_mult
		if (speed<target_speed*speed_mult):
			speed+=delta*ACCEL
		if (speed>target_speed*speed_mult):
			speed=target_speed*speed_mult
	
	if (current_powerup==POWERUP_BALLOON):
		if (get_global_pos().y<-400):
			
			set_global_pos( Vector2(get_global_pos().x,-400) )
			vvelocity=0
			#current_powerup=POWERUP_NONE
			#base_anim=""
			
		else:
			if (not mouse_pressing):
				vvelocity+=delta*BALLOON_FALL
			else:
				vvelocity+=delta*BALLOON_LIFT
	else:
		vvelocity+=delta*GRAVITY
	
#	if (current_powerup==POWERUP_UMBRELLA and mouse_pressing):

	if (current_powerup==POWERUP_NONE): 
		if ((not jump_press and mouse_pressing and air_time>MOUSE_MIN_AIR_TIME)):
			if (not using_umbrella):
				get_node("anim").play("umbrella_open")
				get_node("sound_fx").play("umbrella_open_2")
				get_node("anim").queue("umbrella_mid_jump")
				using_umbrella=true
				get_node("/root/gamestate").p_info.data.total_umbrellas+=1
				if (get_node("/root/gamestate").p_info.data.total_umbrellas==Config.ACHIEVEMENT_FLOATER):
					get_node("/root/gamestate").set_achievement_unlocked("random_umbrella")
		

		elif(not mouse_pressing and air_time>MOUSE_MIN_AIR_TIME and using_umbrella):
			using_umbrella=false
			get_node("anim").play("umbrella_close")
			get_node("anim").queue("mid_jump")

		if (using_umbrella):			
			var fall_vel = FALL_UMBRELLA_VELOCITY
			if (hat_name=="flying"):
				fall_vel*=0.8
			vvelocity = min( vvelocity, fall_vel )
	
	else:
		using_umbrella=false		
			
		
		
#	if (wind_count>0 and current_powerup==POWERUP_UMBRELLA):		
#		vvelocity=JUMP_UMBRELLA_VELOCITY
	if (wind_count>0 && using_umbrella):		
		vvelocity=JUMP_UMBRELLA_VELOCITY
		
	var velocity = Vector2(speed,vvelocity)
	
	var motion = velocity * delta
	var desired_target_x = get_global_pos().x+motion.x
	var prev_pos_x = get_global_pos().x
	
	motion = move(motion)
	

	var on_floor = false
	
	if (is_colliding()):
		#ran against something, is it the floor? get normal
		var n = get_collision_normal()
		

		if ( rad2deg(acos(n.dot( Vector2(0,-1)))) < FLOOR_ANGLE_TOLERANCE ):
			if (air_time>MAX_ONFLOOR_TIME and current_powerup!=POWERUP_BALLOON and current_powerup!=POWERUP_CAR):
				get_node("anim").play(base_anim+"land")
				get_node("anim").queue(base_anim+"run")
				
			on_floor=true
			
			
			air_time=0
			#if angle to the "up" vectors is < angle tolerance
			#char is on floor
#			on_air_time=0
#			floor_velocity=get_collider_velocity()
			#velocity.y=0 
			
		# But we were moving and our motion was interrupted, 
		# so try to complete the motion by "sliding"
		# by the normal

	
		motion = n.slide(motion)
		velocity = n.slide(velocity)
		vvelocity=velocity.y
		
		#then move again
		move(motion)
		
		#set_global_pos(Vector2(desired_target_x,get_global_pos().y))
		
	
		
	air_time+=delta	
	jump_air_time+=delta
	
	if (current_powerup!=POWERUP_BALLOON and current_powerup!=POWERUP_CAR):
	
		if (not jumping and air_time-delta<MAX_ONFLOOR_TIME && air_time>=MAX_ONFLOOR_TIME):
			if (not using_umbrella):
				get_node("anim").play(base_anim+"begin_jump")
			umbrella_fall=true
			
#		if (not on_floor and not jumping and velocity.y>0 and mouse_pressed and current_powerup==POWERUP_UMBRELLA):
#			get_node("anim").play("umbrella_land")
#			umbrella_fall=false
			
		if (not jumping and not on_floor and mouse_pressed):
			jump_air_time=0
			
		"""
		if (mouse_pressed):
			print("NOT DJ: ", disable_jump)
			print("NOT J: ", not jumping)
			print("OF: ", on_floor)
			print("OF: ", on_floor)
			print("JBEF: ",(mouse_pressed or jump_air_time<MOUSE_MIN_AIR_TIME))
			print("JAFT: ",(mouse_pressed and not on_floor and air_time<MOUSE_MIN_AIR_TIME))
		"""
		if (not disable_jump and not jumping and ((on_floor and (mouse_pressed or jump_air_time<MOUSE_MIN_AIR_TIME)) or (mouse_pressed and not on_floor and air_time<MOUSE_MIN_AIR_TIME) )):
			if (current_powerup==POWERUP_POGO):
				vvelocity=POGO_JUMP_VELOCITY
				get_node("sound_fx").play("pogo_"+str(randi()%4+1))
			else:
				var jump_vel = JUMP_VELOCITY
				#if (hat_name=="mariachi"): DEPRECATED !!!!
				#	jump_vel*=1.2
				vvelocity=jump_vel
				get_node("sound_fx").play("whoosh_jump_2")
			jumping=true
			jump_press=true
			umbrella_fall=true
			if (not using_umbrella):			
				get_node("anim").play(base_anim+"begin_jump")
			jump_air_time=MOUSE_MIN_AIR_TIME*2.0
			
		if (jumping and vvelocity>0):
			jumping=false
			if (not using_umbrella):
				get_node("anim").play(base_anim+"mid_jump")
		
		
		if (jumping and velocity.y<0):
			if (not mouse_pressing):
				velocity.y+=JUMP_STOP_FORCE*delta
				vvelocity=velocity.y
			
			
	var dstep = ((get_global_pos().x-prev_pos_x)/Config.BEAN_DISTANCE_DIVIDER)
	distance_run+=dstep
	var tmr = get_node("/root/gamestate").p_info.data.total_meters_run
	if (not level_ended and tmr<Config.ACHIEVEMENT_METERS_RUN and tmr+dstep>=Config.ACHIEVEMENT_METERS_RUN):
		get_node("/root/gamestate").set_achievement_unlocked("bean_marathon")
		
	get_node("/root/gamestate").p_info.data.total_meters_run=tmr+dstep
	

	if (not level_ended and air_time>MOUSE_MIN_AIR_TIME):
		var tjt = get_node("/root/gamestate").p_info.data.total_jump_time
		if (tjt<Config.ACHIEVEMENT_JUMP_TIME and tjt+dstep>=Config.ACHIEVEMENT_JUMP_TIME):
			get_node("/root/gamestate").set_achievement_unlocked("time_jumping")			
		get_node("/root/gamestate").p_info.data.total_jump_time=tjt+dstep

	get_node("hudbase/hud").set_distance(distance_run)
	mouse_pressed=false

func slowdown():		
		speed=SLOWDOWN_SPEED*speed_mult	
var damage_count=0	
			
func damage(soft=false):
	if (stopped):
		return
		
	damage_count+=1
	
	if (invincible_time>0 or untrip_time>0):
		return
	
	untrip_time = UNTRIP_MAX
	
	if (soft):
		get_node("sound_fx").play("punch")
	else:
		get_node("sound_fx").play("crash_"+str(randi()%4+1))
	
	if (current_powerup==POWERUP_CAR):
#		print(damage_count," HIT car powerup")
		return	
#	print(damage_count," HIT cno powerup")
			
			
	get_node("shake").play("shake")
	get_node("hit").set_emitting(true)
	if (current_powerup):	
		powerup_chance-=1
		if (powerup_chance==0):		
			current_powerup=POWERUP_NONE
			get_node("car_fx").stop_all()
			base_anim=""
			jumping=false

	if (current_powerup==POWERUP_NONE):
		
		if (air_time<MAX_ONFLOOR_TIME):
				get_node("anim").play("hit")
				get_node("anim").queue("run")
		else:
				get_node("anim").play("airhit")
				if (using_umbrella):
					get_node("anim").queue("umbrella_mid_jump")
				else:
					get_node("anim").queue("mid_jump")
		speed=SLOWDOWN_SPEED*speed_mult
	
		
	get_node("smoke").set_emitting(false)
	get_node("frames/hat").show()
	
	
func goo_damage(type):
	if (stopped):
		return
		
	if (current_powerup==POWERUP_CAR or invincible_time>0 or untrip_time>0):
			return	
			
	get_node("shake").play("shake")
	get_node("goo_"+type).set_emitting(true)
	if (current_powerup):	
		powerup_chance-=1
		if (powerup_chance==0):		
			current_powerup=POWERUP_NONE
			get_node("car_fx").stop_all()
			base_anim=""

	if (current_powerup==POWERUP_NONE):
		if (air_time<MAX_ONFLOOR_TIME):
				get_node("anim").play("hit")
				get_node("anim").queue("run")
		else:
				get_node("anim").play("airhit")
				get_node("anim").queue("mid_jump")
		speed=SLOWDOWN_SPEED*speed_mult
	
		
	get_node("smoke").set_emitting(false)
	get_node("frames/hat").show()
	
	get_node("sound_fx").play("splotch")
	untrip_time=UNTRIP_MAX
	
	

func hit():
	get_node("hit").set_emitting(true)


func gameover():
	get_node("hudbase/hud").show_gameover()
	invincible_time=INVINCIBLE_MAX

func show_tutorial(idx):
	get_node("hudbase/hud").show_tutorial(idx)

func set_disable_jump(disable):
	disable_jump=disable
func make_it_jump():
	vvelocity=JUMP_VELOCITY
func fill_bean_rush():
	if (bean_rush_count==0):
		bean_rush_count+=1		
		get_node("hudbase/hud").set_bean_rushes(bean_rush_count)

func hit_victim():
	get_node("shake").play("hshake")
	get_node("hit").set_emitting(true)
	
	if (current_powerup!=POWERUP_CAR and air_time<MAX_ONFLOOR_TIME):
		get_node("anim").play("hit2")
		get_node("anim").queue("run")
		
	get_node("/root/gamestate").p_info.data.total_mishaps+=1
	if (get_node("/root/gamestate").p_info.data.total_mishaps==Config.ACHIEVEMENT_MISHAPS1):
		get_node("/root/gamestate").set_achievement_unlocked("mishap_madness")

	if (get_node("/root/gamestate").p_info.data.total_mishaps==Config.ACHIEVEMENT_MISHAPS2):
		get_node("/root/gamestate").set_achievement_unlocked("random_angry_mobs")
		
	get_node("sound_fx").play("punch")
	
func _cat_grab():
	get_node("catfly").stop()
	get_node("anim").play("catted")
	get_node("sound_fx").play("Big_ow")
	get_node("hit").set_emitting(true)
	stopped=true
	
func enable_powerup(idx):

	if (current_powerup==POWERUP_CAR):
		return #no changy
	current_powerup=idx
	get_node("smoke").set_emitting(false)
	get_node("frames/hat").show()
	powerup_time=powerup_times[idx]
	powerup_chance=powerup_chances[idx]
	
	if (idx==POWERUP_BALLOON):
		get_node("anim").play("balloon_run")
		get_node("/root/gamestate").p_info.data.total_balloons+=1
		vvelocity=0.0 # reset vertical velocity
		if (get_node("/root/gamestate").p_info.data.total_balloons>=Config.ACHIEVEMENT_HOT_AIR):
			get_node("/root/gamestate").set_achievement_unlocked("random_hot_air")
		
		get_node("sound_fx").play("Balloons_Yippee")
		
		
	if (idx==POWERUP_POGO):
		base_anim="pogo_"	
		get_node("/root/gamestate").p_info.data.total_pogos+=1
		if (get_node("/root/gamestate").p_info.data.total_pogos>=Config.ACHIEVEMENT_POGO):
			get_node("/root/gamestate").set_achievement_unlocked("random_bouncing_bean")
		get_node("sound_fx").play("boing")
	if (idx==POWERUP_CAR):
		base_anim="car_"
		get_node("smoke").set_emitting(true)
		car_crashes=MAX_CAR_CRASHES
		get_node("frames/hat").hide()
		get_node("anim").play("car_run")
		if get_node("/root/globals").global_powerups["trucker"]:
			powerup_time*=1.5
			
		### LONGER CAR BOOST FROM AD
		if (get_node("/root/globals").longer_car_boost):
			powerup_time*=float(get_node("/root/gamestate").get_app_parameter("heyzap_carboost",int(Config.HEYZAP_CARBOOST)))
			
		get_tree().call_group(0,"wicket","go_back_fast")

func random_powerup():
	enable_powerup(1+(randi() % (POWERUP_MAX-1)))
	

func _bean_rush():
	if (blocked or bean_rush_count==0 or current_powerup==POWERUP_CAR or current_powerup==POWERUP_BALLOON):
		return
	bean_rush_count-=1
	get_node("hudbase/hud").set_bean_rushes(bean_rush_count)
	get_node("/root/gamestate").p_info.data.total_rushes+=1
	enable_powerup(POWERUP_CAR)
	speed=CAR_TARGET_SPEED
	bean_count_br=bean_count
	
	if (get_node("/root/gamestate").p_info.data.total_rushes>=Config.ACHIEVEMENT_RUSH):
		get_node("/root/gamestate").set_achievement_unlocked("random_bean_rush")
	get_node("car_fx").play("carloop")		
	get_tree().call_group(0,"angrymob","disable_victims")
	get_tree().call_group(0,"wicket","calmdown")
	vvelocity=0;
	get_node("hudbase/hud").set_bean_rush_fill(0)
	get_node("/root/globals").bean_rush_fill=0
	get_node("/root/gamestate").set_bean_rush(bean_rush_count)	
	
	
func coin(big):

	if (big):
		for i in range(Config.COIN_BIG_VALUE):
			coin(false)
		return
	else:
		if get_node("/root/globals").global_powerups["golden"]:
			bean_count+=2
		else:
			bean_count+=1
		
	get_node("hudbase/hud").set_beans(bean_count)
	get_node("/root/globals").collected_beans_endless = bean_count
	
	if (bean_rush_count==0):
		var bc_dif = bean_count-bean_count_br
		var coins_per_br = COINS_PER_BEAN_RUSH
		
		if get_node("/root/globals").global_powerups["sports"]:
			coins_per_br -= (COINS_PER_BEAN_RUSH * 
							(int(get_node("/root/gamestate").get_app_parameter("powerup_sports",Config.POWERUP_SPORTS)))
							/ 100)
		
		if (bc_dif>coins_per_br):
			bean_rush_count+=1
			get_node("hudbase/hud").set_bean_rushes(bean_rush_count)
			get_node("/root/gamestate").set_bean_rush(bean_rush_count)	
			get_node("/root/globals").bean_rush_fill=0
			
		else:	
			get_node("hudbase/hud").set_bean_rush_fill(bc_dif/float(coins_per_br))
			get_node("/root/globals").bean_rush_fill=bc_dif
			
		
	get_node("coin_fx").play("pickup_"+str(randi()%7+1))
		
	
func take_teddy():
	get_node("hudbase/hud").show_teddy()
	get_node("sound_fx").play("Teddy")

func take_gift():
	get_node("hudbase/hud").show_yellow_gift()

func _continue():
	print("bean continue")
	stopped=false
	get_node("cat").hide()
	get_node("smacked").set_emitting(false)
	get_node("dizzy").set_emitting(false)
	get_node("anim").play("run")
	jumping=false
	blocked=false
	
	emit_signal("continued")

func finished():
	current_powerup=POWERUP_NONE
	get_node("car_fx").stop_all()
	base_anim=""
	level_ended=true
	jumping=false
	get_node("camera").clear_current()
	get_node("/root/gamestate").set_beans( get_node("/root/gamestate").get_beans() + bean_count )
	get_node("hudbase/hud").hide_hud()
	emit_signal("finished")
	


func wicket_hit(usecat):
	if (usecat):
		if get_node("/root/globals").global_powerups["construction"] && !first_hit:
			get_node("catfly").play("catfail")
			first_hit = true
			return false
		else:
			get_node("catfly").play("catfly")
			blocked=true
			
		get_node("sound_fx").play("cat_"+str(randi()%4+1))
	else:
		if get_node("/root/globals").global_powerups["construction"] && !first_hit:
			if get_node("/root/gamestate").p_info.data["hats"]["construction"] == "unequip":
				_hat_fall()
			first_hit = true
			return false
		else:
			get_node("anim").play("ko")
			get_node("shake").play("shake")		
			get_node("sound_fx").play("Big_ow")
			stopped=true
			blocked=true

	return true
var wind_count=0

func wind_enter():
	wind_count+=1
	
func wind_exit():
	wind_count-=1
	
func set_speed_mult(mult):
	speed_mult=mult

						
func _ready():

	
	hat=get_node("frames/hat")
	globals = get_node("/root/globals")
	var gstate = get_node("/root/gamestate")
	set_fixed_process(true)
	set_process(true)
	set_process_input(true)
	
	bean_rush_count=gstate.get_bean_rush()
	get_node("hudbase/hud").set_bean_rushes(bean_rush_count)
	
	hat_name = get_node("/root/gamestate").get_current_hat()
	
	if (hat_name == "golden" || hat_name == "flying_nun" || 
	   hat_name == "mexican" || hat_name == "sports" || hat_name == "top"):
		hat_name = { "sports":"basketball", 
					 "mexican":"mariachi", 
					 "flying_nun":"flying", 
					 "golden":"golden_tophat", 
					 "top":"tophat"}[hat_name]
	
	if (hat_name!="no_hat"):
		print ("hat name", hat_name)
		hat.set_sprite_frames( load( "res://bean/hat_"+hat_name+"/hat_frames.res") )
	else:
		hat.set_sprite_frames(null)
		
	get_node("hudbase/hud").connect("do_bean_rush",self,"_bean_rush")
	get_node("hudbase/hud").connect("continue",self,"_continue")
	get_node("/root/endless").connect("continue",self,"_continue")
	get_node("/root/endless").connect("do_bean_rush",self,"_bean_rush")
	get_node("/root/endless").connect("update_bean_rush",self,"update_bean_rush_count")

	get_node("frames").connect("draw",self,"_update_hat")
	# Initalization here
	
#	if (get_node("/root/globals").bean_rush_stored):	
#		bean_rush_count+=1
#		get_node("hudbase/hud").set_bean_rushes(bean_rush_count)
#		get_node("/root/globals").bean_rush_stored=true
		
	#if (hat_name=="basketball"): DEPRECATED !!!!
	#	target_speed*=1.2
	#elif (hat_name=="bowler"): 
	#	target_speed*=0.9
		
	if !get_node("/root/globals").global_powerups["top"]:
		get_node("magnet").queue_free()
		print("remove top hat")
		
	bean_count_br = -get_node("/root/globals").bean_rush_fill
	

		
	#app parameters
	GRAVITY = float(gstate.get_app_parameter("bean_gravity",GRAVITY))
	JUMP_VELOCITY = float(gstate.get_app_parameter("bean_jump_velocity",JUMP_VELOCITY))
	ACCEL = float(gstate.get_app_parameter("bean_accel",ACCEL))
	TARGET_SPEED = float(gstate.get_app_parameter("bean_max_speed",TARGET_SPEED))
	SLOWDOWN_SPEED = float(gstate.get_app_parameter("bean_min_speed",SLOWDOWN_SPEED))
	powerup_times[POWERUP_CAR]=float(gstate.get_app_parameter("bean_boost_duration",powerup_times[POWERUP_CAR]))
	COINS_PER_BEAN_RUSH = int(gstate.get_app_parameter("beans_per_boost",COINS_PER_BEAN_RUSH))
	pass

func update_bean_rush_count():
	bean_rush_count = get_node("/root/gamestate").get_bean_rush()
	get_node("hudbase/hud").set_bean_rushes(bean_rush_count)

func set_enable_endless(base_dist):
	get_node("hudbase/hud").set_enable_endless()
	get_node("hudbase/hud").set_distance(base_dist)
	distance_run=base_dist
	
func get_distance_run():
	return distance_run

func set_current_level(idx):
	get_node("hudbase/hud").set_current_level(idx)



func _init():
	add_user_signal("continued")
	add_user_signal("finished")

func _hat_fall():
	get_node("hatfall_anim").play("hatfall")
	get_node("shake").play("shake")		
	hat_name=""
	pass
	