
extends TextureFrame

var v_scroll = 0
var r_dif = 0.0
var max_scroll = 0
var down_scroll_times = 0
var last_node_dif = 0
var row_minimum_size_y = 0
var max_scroll_times = 0
var division_remainder = 0 
var max_column_size = 0
var up_scroll_times = 0

## LEADERBOARD CACHE
var cached_friends_ranking = {}
var cached_friends_leaderboard = {}
var cached_global_ranking = {}
var cached_global_leaderboard = {}
var leaderboard_timer = null
var current_country = null

var receivedRanking=false
var receivedLearderboard=false

var countries_frame_count = 0

var server

var template_row

var isGlobalLeaderboard = false

var first_place = preload("res://screens/main_menu/leaderboard/sprites/star_first.png")
var second_place = preload("res://screens/main_menu/leaderboard/sprites/star_second.png")
var third_place = preload("res://screens/main_menu/leaderboard/sprites/star_third.png")

var sprite_row = preload("res://screens/main_menu/leaderboard/sprite_row.scn")
var label_row = preload("res://screens/main_menu/leaderboard/label_row.scn")

func _ready():
	self.add_user_signal("backed")
	server = get_node("/root/gamestate")
	server.connect("leaderboard_ok",self,"set_leaderboard_rows")
	server.connect("ranking_ok",self,"set_player_ranking")
	server.connect("leaderboard_offline",self,"leaderboard_offline")
	server.connect("user_logged_out",self,"disable_friends")
	server.connect("user_logged_in",self,"enable_friends")
	get_node("scoreboard/scorelist").get_node("_v_scroll").set_opacity(0)
	countries_frame_count = get_node("countries").get_sprite_frames().get_frame_count("default")
	get_node("cache_timeout").connect("timeout",self,"leaderboard_cache_timeout")
	
	if(OS.get_locale() == "de_DE" || OS.get_locale().find("de") != -1):
		get_node("country_label").set_scale(Vector2(0.7,0.7))
		get_node("score_container/score").set_scale(Vector2(0.95,0.95))
	
	if(server.isLoggedin):
		get_node("share_container/share_btn").set_disabled(false)
		get_node("share_container/share_btn").set_pressed(false)


func enable_friends():
	clear_leaderboard()
	get_node("cache_timeout").stop()
	leaderboard_cache_timeout()
	if(get_node("share_container/share_btn").is_disabled()):
		get_node("share_container/share_btn").set_disabled(false)
	if(get_node("share_container/share_btn").is_pressed()):
		get_node("share_container/share_btn").set_pressed(false)

func disable_friends():
	clear_leaderboard()
	get_node("cache_timeout").stop()
	leaderboard_cache_timeout()
	if(!get_node("share_container/share_btn").is_pressed()):
		get_node("share_container/share_btn").set_pressed(true)
	
	if(!get_node("share_container/share_btn").is_disabled()):
		get_node("share_container/share_btn").set_disabled(true)

func leaderboard_offline():
	if(get_node("loading_screen").is_visible()):
		get_node("loading_screen").hide()

	if(get_node("scoreboard/leaderboard_offline").is_hidden()):
		get_node("scoreboard/leaderboard_offline").show()

func set_player_ranking(ranking):
	receivedRanking=true
	if(get_node("loading_screen").is_visible() && receivedLearderboard):
		get_node("loading_screen").hide()
	get_node("score_container/player_ranking").set_text(str(ranking[0]))
	get_node("score_container/player_score").set_text(str(server.p_info.leaderboard[server.p_info.locations[get_node("countries").get_frame()] + "_highscore"])+"km")

func set_leaderboard_rows(list):
	receivedLearderboard=true
	if(get_node("loading_screen").is_visible() && (receivedRanking || !server.isLoggedin)):
		get_node("loading_screen").hide()
	if(!get_node("scoreboard/leaderboard_offline").is_hidden()):
		get_node("scoreboard/leaderboard_offline").hide()

	var container = "scoreboard/scorelist/score_rows"
	if(list.size() == 0):
		print("something went wrong")
		return
	elif(list[0].size()==0):
		print("No record found")
		return
		
	if(get_node("cache_timeout").get_time_left() <= 0):
		get_node("cache_timeout").start()
		print("STARTED CACHE LEADERBOARD")
	
	for i in range(list[0].size()):
		if(i == 0):
			create_new_row(container,list[0][i][0],list[0][i][1],first_place,i)
		elif(i == 1):
			create_new_row(container,list[0][i][0],list[0][i][1],second_place,i)
		elif(i ==2 ):
			create_new_row(container,list[0][i][0],list[0][i][1],third_place,i)
		else:
			create_new_row(container,list[0][i][0],list[0][i][1],null,i)
	
	set_row_dif_and_max_scroll(container)

func _on_back_btn_pressed():
	if(self.is_visible()):
		self.hide()
		self.emit_signal("backed")

func _on_main_menu_btn_pressed():
	if(self.is_visible()):
		self.hide()
		self.emit_signal("backed")
		if(self.get_parent().get_node("bottom/menu_ok").is_visible()):
			self.get_parent().get_node("bottom/menu_ok").hide()
			self.get_parent().get_node("bottom/play_btn").show()
			self.get_parent().get_node("bottom/menu_btn").show()
			if(self.get_parent().get_node("bottom/back_btn").is_visible()):
				self.get_parent().get_node("bottom/back_btn").hide()

func _on_top_arrow_btn_pressed():
	if(get_node("scoreboard/scorelist").get_v_scroll() > 0):
		v_scroll -= max_scroll
		get_node("scoreboard/scorelist").set_v_scroll(v_scroll)
		
		if(up_scroll_times <= max_scroll_times):
			up_scroll_times += 1
			down_scroll_times = max_scroll_times - up_scroll_times

func _on_bottom_arrow_btn_pressed():	
	if(up_scroll_times > 0):
		up_scroll_times = 0

	if(max_scroll_times == 0 && division_remainder == 0):
		return
		
	down_scroll_times += 1

	if((v_scroll < max_column_size) && (down_scroll_times <= max_scroll_times || max_scroll_times == 0)):
		v_scroll += max_scroll
		
		if((down_scroll_times == max_scroll_times || max_scroll_times == 0) && r_dif > 0):
			down_scroll_times = 0
			get_node("scoreboard/scorelist").set_v_scroll(v_scroll - last_node_dif)
		else:
			get_node("scoreboard/scorelist").set_v_scroll(v_scroll)

func _on_right_arrow_btn_pressed():
	if(get_node("countries").get_frame() == (countries_frame_count - 1) ):
		set_frame_and_get_leaderboard(0)
	else:
		set_frame_and_get_leaderboard(get_node("countries").get_frame() + 1)
		
func _on_left_arrow_btn_pressed():
	if(get_node("countries").get_frame() == 0):
		set_frame_and_get_leaderboard(countries_frame_count - 1)
	else:
		set_frame_and_get_leaderboard(get_node("countries").get_frame() - 1)

func set_row_dif_and_max_scroll(node):
	division_remainder = get_node("scoreboard/scorelist/score_rows").get_child_count()%4
	var number_of_clicks = get_node("scoreboard/scorelist/score_rows").get_child_count()/4
	
	if(number_of_clicks <= 2):
		max_scroll_times = number_of_clicks - 1
	else:
		if(division_remainder == 0):
			max_scroll_times = number_of_clicks - 1
		else:
			max_scroll_times = number_of_clicks
	
	var childrens = get_node(node).get_children()
	if(childrens.size() > 0):
		row_minimum_size_y = childrens[0].get_node("place").get_custom_minimum_size().y
		max_scroll = (childrens[0].get_custom_minimum_size().y + 15) * 4
		max_column_size = (max_scroll * (get_node("scoreboard/scorelist/score_rows").get_child_count()/4)) - (row_minimum_size_y/2)
		
	if(row_minimum_size_y <= 0):
		return
	
	if(get_node(node).get_child_count() % 4 != 3):
		r_dif = (get_node(node).get_child_count() % 4) * (row_minimum_size_y + 15)
	else:
		r_dif = (get_node(node).get_child_count() % 4) * row_minimum_size_y
	
	if((get_node(node).get_child_count() % 4) == 1):
		last_node_dif = max_scroll - r_dif
	elif((get_node(node).get_child_count() % 4) == 3):
		last_node_dif = row_minimum_size_y + 15
	else:
		last_node_dif = r_dif
	
func set_frame_and_get_leaderboard(frame):
	receivedLearderboard=false
	receivedRanking=false
	
	if(countries_frame_count != server.p_info.locations.size()):
		print("invalid frames size")
		return
	
	clear_leaderboard()
	get_node("score_container/player_ranking").set_text("")
	
	if(get_node("loading_screen").is_hidden()):
		get_node("loading_screen").show()
		
	get_node("countries").set_frame(frame)
	var map = {"uk":"uk","india":"india","japan":"jap","australia":"aus","mexico":"mex","usa":"us"}[server.p_info.locations[get_node("countries").get_frame()]]
	get_node("country_label").set_text(tr("LEAD_ENDLESS_" + map.to_upper()))
	if(server.parse):
		if(isGlobalLeaderboard):
			server.get_global_leaderboard(server.p_info.locations[get_node("countries").get_frame()])
		else:
			server.get_friends_leaderboard(server.p_info.locations[get_node("countries").get_frame()])

func clear_leaderboard():
	var c_node = "scoreboard/scorelist/score_rows"
	if(get_node(c_node).get_child_count() > 0):
		for i in range(get_node(c_node).get_child_count()-1,-1,-1):
			get_node(c_node).get_child(i).queue_free()
	
	print("Cleared leaderboard !!")

func create_new_row(container,player_name,player_score, spriteTexture,i):
	var new_row

	if(spriteTexture):
		new_row = sprite_row.instance()
		new_row.get_node("place").set_texture(spriteTexture)
	else:
		new_row = label_row.instance()
		new_row.get_node("place").set_text("0"+str(i+1)+".")
	
	new_row.get_node("player_name").set_text(player_name)
	new_row.get_node("player_score").set_text(str(player_score)+ "km")
	get_node(container).add_child(new_row)

func _on_share_btn_pressed():
	if(get_node("share_container/share_btn").is_pressed()):
		isGlobalLeaderboard = true
		set_frame_and_get_leaderboard(get_node("countries").get_frame())
	else:
		isGlobalLeaderboard = false
		set_frame_and_get_leaderboard(get_node("countries").get_frame())
		
func leaderboard_cache_timeout():
	server.cached_friends_leaderboard.clear()
	server.cached_friends_ranking.clear()
	server.cached_global_leaderboard.clear()
	server.cached_global_ranking.clear()